def cbrt(x):
    from .constants import pi
    from .sqrt import sqrt
    from .atan import atan
    from .exp import exp
    if type(x) is complex:
        a = x.real
        b = x.imag
        if a != 0:
            r = sqrt(a**2+b**2)
            if a > 0 and b > 0:
                theta = atan(b/a)
            elif a > 0 and b < 0:
                theta = 2*pi-atan(-b/a)
            elif a < 0 and b > 0:
                theta = pi - atan(b/-a)
            elif a < 0 and b < 0:
                theta = pi + atan(-b/-a)
            elif a > 0 and b == 0:
                theta = 0
            else:
                theta = 3*pi
            low = 0.0
            high = max(1.0, r)
            epsilon = 0.00000000000001
            while True:
                mid = (low + high)/2
                mid_cubed = mid * mid * mid

                if abs(mid_cubed-r) < epsilon:
                    cubed_r = mid
                    break
                elif mid_cubed < r:
                    low = mid
                else:
                    high = mid
            principal_root = cubed_r * exp(theta*1j/3)
            if abs(principal_root.imag) < 1e-8:
                return principal_root.real
            return principal_root
            
        if b >= 0:
            low = 0.0
            high = max(1.0, b)
        else:
            low = min(-1.0, b)
            high = 0.0
        
        epsilon = 0.00000000000001
    
        while True:
            mid = (low + high) / 2
            mid_cubed = mid * mid * mid

            if abs(mid_cubed - b) < epsilon:
                cubed_b = mid
                break
            elif mid_cubed < b:
                low = mid
            else:
                high = mid
        return cubed_b*-1j
    if x >= 0:
        low = 0.0
        high = max(1.0, x)
    else:
        low = min(-1.0, x)
        high = 0.0
    
    epsilon = 1e-10
    
    while True:
        mid = (low + high) / 2
        mid_cubed = mid * mid * mid

        if abs(mid_cubed - x) < epsilon:
            return mid
        elif mid_cubed < x:
            low = mid
        else:
            high = mid



def quartic(a,b,c,d,e):
    from .sqrt import sqrt
    from .cubic import cubic
    if a == 0:
        raise ValueError("Value of 'a' must be different to 0.")
    if e == 0:
        cubic(a,b,c,d)
        quartic.x1, quartic.x2, quartic.x3, quartic.x4 = 0, cubic.x1, cubic.x2, cubic.x3
    elif a+b+c+d+e == 0:
        A = a
        B = b+A
        C = c+B
        D = -e
        cubic(A,B,C,D)
        quartic.x1, quartic.x2, quartic.x3, quartic.x4 = 1, cubic.x1, cubic.x2, cubic.x3
    elif a-b+c-d+e == 0:
        A = a
        B = b-A
        C = c-B
        D = e
        cubic(A,B,C,D)
        quartic.x1, quartic.x2, quartic.x3, quartic.x4 = -1, cubic.x1, cubic.x2, cubic.x3
    else:
        p = (8*a*c-3*b**2)/(8*a**2)
        q = (b**3-4*a*b*c+8*a**2*d)/(8*a**3)
        r = (16*a*c*b**2-3*b**4-64*a**2*d*b+256*a**3*e)/(256*a**4)
        cubic(1,5*p/2,2*p**2-r,(4*p**3-4*p*r-q**2)/8)
        z = cubic.x1
        quartic.x1 = (sqrt(p+2*z)+sqrt(-3*p-2*z-2*q/sqrt(p+2*z)))/2-b/(4*a)
        quartic.x2 = (sqrt(p+2*z)-sqrt(-3*p-2*z-2*q/sqrt(p+2*z)))/2-b/(4*a)
        quartic.x3 = (-sqrt(p+2*z)+sqrt(-3*p-2*z+2*q/sqrt(p+2*z)))/2-b/(4*a)
        quartic.x4 = (-sqrt(p+2*z)-sqrt(-3*p-2*z+2*q/sqrt(p+2*z)))/2-b/(4*a)
    pre_result = [quartic.x1, quartic.x2, quartic.x3, quartic.x4]
    result = []
    for solution in pre_result:
        if solution not in result:
            result.append(solution)
    return result